=== modified file 'breezy/bzr/dirstate.py'
--- breezy/bzr/dirstate.py	2017-07-23 17:23:58 +0000
+++ breezy/bzr/dirstate.py	2017-08-26 13:06:15 +0000
@@ -3358,6 +3358,7 @@
         self._lock_state = 'r'
         self._state_file = self._lock_token.f
         self._wipe_state()
+        return lock.LogicalLockResult(self.unlock)
 
     def lock_write(self):
         """Acquire a write lock on the dirstate."""
@@ -3371,6 +3372,7 @@
         self._lock_state = 'w'
         self._state_file = self._lock_token.f
         self._wipe_state()
+        return lock.LogicalLockResult(self.unlock)
 
     def unlock(self):
         """Drop any locks held on the dirstate."""
@@ -3393,7 +3395,7 @@
 
 
 def py_update_entry(state, entry, abspath, stat_value,
-                 _stat_to_minikind=DirState._stat_to_minikind):
+                    _stat_to_minikind=DirState._stat_to_minikind):
     """Update the entry based on what is actually on disk.
 
     This function only calculates the sha if it needs to - if the entry is

=== modified file 'breezy/decorators.py'
--- breezy/decorators.py	2017-05-22 00:56:52 +0000
+++ breezy/decorators.py	2017-08-26 13:06:15 +0000
@@ -99,17 +99,8 @@
     #       foo.func_code.co_argcount and foo.func_code.co_varnames
     template = """\
 def %(name)s_read_locked(%(params)s):
-    self.lock_read()
-    try:
-        result = unbound(%(passed_params)s)
-    except:
-        try:
-            self.unlock()
-        finally:
-            raise
-    else:
-        self.unlock()
-        return result
+    with self.lock_read():
+        return unbound(%(passed_params)s)
 read_locked = %(name)s_read_locked
 """
     params, passed_params, defaults_dict = _get_parameters(unbound)
@@ -143,17 +134,8 @@
             stuff
     """
     def read_locked(self, *args, **kwargs):
-        self.lock_read()
-        try:
-            result = unbound(self, *args, **kwargs)
-        except:
-            try:
-                self.unlock()
-            finally:
-                raise
-        else:
-            self.unlock()
-            return result
+        with self.lock_read():
+            return unbound(self, *args, **kwargs)
     read_locked.__doc__ = unbound.__doc__
     read_locked.__name__ = unbound.__name__
     return read_locked
@@ -163,17 +145,8 @@
     """Decorate unbound to take out and release a write lock."""
     template = """\
 def %(name)s_write_locked(%(params)s):
-    self.lock_write()
-    try:
-        result = unbound(%(passed_params)s)
-    except:
-        try:
-            self.unlock()
-        finally:
-            raise
-    else:
-        self.unlock()
-        return result
+    with self.lock_write():
+        return unbound(%(passed_params)s)
 write_locked = %(name)s_write_locked
 """
     params, passed_params, defaults_dict = _get_parameters(unbound)
@@ -196,17 +169,8 @@
 def _fast_needs_write_lock(unbound):
     """Decorate unbound to take out and release a write lock."""
     def write_locked(self, *args, **kwargs):
-        self.lock_write()
-        try:
-            result = unbound(self, *args, **kwargs)
-        except:
-            try:
-                self.unlock()
-            finally:
-                raise
-        else:
-            self.unlock()
-            return result
+        with self.lock_write():
+            return unbound(self, *args, **kwargs)
     write_locked.__doc__ = unbound.__doc__
     write_locked.__name__ = unbound.__name__
     return write_locked

=== modified file 'breezy/inter.py'
--- breezy/inter.py	2017-07-10 23:56:33 +0000
+++ breezy/inter.py	2017-08-26 13:06:15 +0000
@@ -19,6 +19,7 @@
 from __future__ import absolute_import
 
 from .errors import BzrError
+from .lock import LogicalLockResult
 
 
 class NoCompatibleInter(BzrError):
@@ -104,6 +105,7 @@
         a read lock and the target a read lock.
         """
         self._double_lock(self.source.lock_read, self.target.lock_read)
+        return LogicalLockResult(self.unlock)
 
     def lock_write(self):
         """Take out a logical write lock.
@@ -112,6 +114,7 @@
         a read lock and the target a write lock.
         """
         self._double_lock(self.source.lock_read, self.target.lock_write)
+        return LogicalLockResult(self.unlock)
 
     @classmethod
     def register_optimiser(klass, optimiser):

=== modified file 'breezy/lockable_files.py'
--- breezy/lockable_files.py	2017-06-10 01:57:00 +0000
+++ breezy/lockable_files.py	2017-08-26 13:06:15 +0000
@@ -16,14 +16,16 @@
 
 from __future__ import absolute_import
 
+from . import (
+    errors,
+    lock,
+    )
 from .lazy_import import lazy_import
 lazy_import(globals(), """
 import warnings
 
 from breezy import (
     counted_lock,
-    errors,
-    lock,
     osutils,
     transactions,
     urlutils,
@@ -278,9 +280,11 @@
         if token is not None:
             raise errors.TokenLockingNotSupported(self)
         self._lock = self._transport.lock_write(self._escaped_name)
+        return lock.LogicalLockResult(self.unlock)
 
     def lock_read(self):
         self._lock = self._transport.lock_read(self._escaped_name)
+        return lock.LogicalLockResult(self.unlock)
 
     def unlock(self):
         self._lock.unlock()

=== modified file 'breezy/memorytree.py'
--- breezy/memorytree.py	2017-08-07 14:41:20 +0000
+++ breezy/memorytree.py	2017-08-26 13:06:15 +0000
@@ -25,6 +25,7 @@
 
 from . import (
     errors,
+    lock,
     mutabletree,
     revision as _mod_revision,
     )
@@ -186,6 +187,7 @@
                 self.branch.lock_read()
                 self._lock_mode = "r"
                 self._populate_from_branch()
+            return lock.LogicalLockResult(self.unlock)
         except:
             self._locks -= 1
             raise
@@ -214,6 +216,7 @@
                 self._populate_from_branch()
             elif self._lock_mode == "r":
                 raise errors.ReadOnlyError(self)
+            return lock.LogicalLockResult(self.unlock)
         except:
             self._locks -= 1
             raise

=== modified file 'breezy/revisiontree.py'
--- breezy/revisiontree.py	2017-06-10 16:36:53 +0000
+++ breezy/revisiontree.py	2017-08-26 13:06:15 +0000
@@ -20,6 +20,7 @@
 
 from . import (
     errors,
+    lock,
     revision,
     tree,
     )
@@ -80,7 +81,7 @@
 
     def lock_read(self):
         self._repository.lock_read()
-        return self
+        return lock.LogicalLockResult(self.unlock)
 
     def __repr__(self):
         return '<%s instance at %x, rev_id=%r>' % (

=== modified file 'breezy/transport/sftp.py'
--- breezy/transport/sftp.py	2017-05-30 19:39:52 +0000
+++ breezy/transport/sftp.py	2017-08-26 13:06:15 +0000
@@ -875,6 +875,10 @@
                 self.path = path
             def unlock(self):
                 pass
+            def __exit__(self):
+                pass
+            def __enter__(self):
+                pass
         return BogusLock(relpath)
 
     def lock_write(self, relpath):

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2017-07-14 20:34:39 +0000
+++ breezy/tree.py	2017-08-26 13:06:15 +0000
@@ -42,6 +42,7 @@
 
 from . import (
     errors,
+    lock,
     osutils,
     )
 from .decorators import needs_read_lock
@@ -563,7 +564,7 @@
 
         :return: A breezy.lock.LogicalLockResult.
         """
-        pass
+        return lock.LogicalLockResult(self.unlock)
 
     def revision_tree(self, revision_id):
         """Obtain a revision tree for the revision revision_id.

